package com.livelike.flutter.livelike_flutter_sdk

import android.content.Context
import android.graphics.drawable.GradientDrawable
import android.os.Handler
import android.os.Looper
import android.view.View
import com.livelike.engagementsdk.widget.timeline.IntractableWidgetTimelineViewModel
import com.livelike.engagementsdk.widget.timeline.WidgetsTimeLineView
import io.flutter.plugin.common.*
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory


class TimeLineViewFactory(
    private val plugin: LivelikeFlutterSdkPlugin,
    private val messenger: BinaryMessenger
) : PlatformViewFactory(StandardMessageCodec.INSTANCE) {
    override fun create(context: Context?, viewId: Int, args: Any?): PlatformView {
        val creationParams = args as Map<String?, Any?>?
        return FlutterPluginTimeLineView(context, plugin, viewId, creationParams, messenger)
    }
}

class FlutterPluginTimeLineView(
    var context: Context?,
    var plugin: LivelikeFlutterSdkPlugin,
    var viewId: Int,
    creationParams: Map<String?, Any?>?,
    messenger: BinaryMessenger
) : PlatformView, MethodChannel.MethodCallHandler {

    private var timeLineView: WidgetsTimeLineView? = null
    private val methodChannel: MethodChannel = MethodChannel(messenger, "timeLineView$viewId")
    private val uiThreadHandler: Handler = Handler(Looper.getMainLooper())
    private val timeLineStreamHandler = LiveLikeFlutterStreamHandler()
    private var timeLineStreamChannel: EventChannel
    private var pluginTimeLineView: PluginTimeLineView

    init {
        methodChannel.setMethodCallHandler(this)
        pluginTimeLineView = PluginTimeLineView(context!!)
        timeLineStreamChannel = EventChannel(messenger, "timeline_stream_$viewId")
        timeLineStreamChannel.setStreamHandler(timeLineStreamHandler)
    }

    override fun getView(): View = pluginTimeLineView

    override fun dispose() {
        timeLineView?.removeAllViews()
        timeLineView = null
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "setSession" -> {
                val programId = call.argument<String>("programId") as String
                val gap = call.argument<Double>("gap") as Double
                plugin.sessionMap[programId]?.let { session ->
                    val viewModel = IntractableWidgetTimelineViewModel(session)
                    timeLineView = WidgetsTimeLineView(
                        context!!,
                        viewModel,
                        plugin.engagementSDK
                    )
                    val form = GradientDrawable()
                    form.setSize(1, gap.toInt())
                    timeLineView?.setSeparator(form)
//                    timeLineView!!.setSeparator(
//                        ContextCompat.getDrawable(
//                            context!!,
//                            R.drawable.white_separator
//                        )
//                    )
                    pluginTimeLineView.removeAllViews()
                    pluginTimeLineView.addView(timeLineView)
                }
                result.success(null)
            }
        }
    }
}