import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk_example/app_widgets.dart';
import 'package:livelike_flutter_sdk_example/main.dart';
import 'package:shared_preferences/shared_preferences.dart';

class HomeScreen extends StatelessWidget {
  final controller = Get.put(HomeController());

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text("Home")),
      body: ListView(
        padding: const EdgeInsets.only(top: 10, bottom: 20),
        children: [
          ListTile(
            title: Text("Platform"),
            subtitle: Obx(() => Text(
                "${controller.platformVersion.value}\n${Platform.version}")),
          ),
          ListTile(
            title: Text("SDK Version"),
            subtitle: Text("0.0.1"),
          ),
          Divider(),
          ListTile(
            title: Row(
              children: [
                Expanded(child: Text("Access Token")),
                Obx(() => controller.accessTokenLoading.value
                    ? SizedBox(
                        child: CircularProgressIndicator(strokeWidth: 1.5),
                        width: 15,
                        height: 15)
                    : SizedBox.shrink()),
              ],
            ),
            subtitle: Obx(() => Text("${controller.accessToken.value}")),
          ),
          Divider(),
          Obx(() => ListTile(
                title: Text("Channel"),
                subtitle: Text(
                    "${controller.selectedChannel.value?.name ?? "Select Channel"}"),
                onTap: () async {
                  final channel =
                      await Get.toNamed(Routes.selectChannel.toString());
                  if (channel != null) {
                    controller.selectedChannel.value = channel;
                  }
                },
              )),
          Divider(),
          ListTile(
            title: Text("All Features Screen"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.allfeatures.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Widget Only"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.widgetOnly.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Public Chat"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.chatOnly.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Chat Session Only"),
            onTap: () {
              Get.toNamed(Routes.chatOnly.toString());
            },
          ),
          Divider(),
          ListTile(
            title: Text("LeaderBoard"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.leaderBoard.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Sponsor"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.sponsor.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Widget Details"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.widgetDetails.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Widget List"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.widgetList.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Json Theme"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.jsonTheme.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Programs"),
            onTap: () {
              Get.toNamed(Routes.programs.toString());
            },
          ),
          Divider(),
          ListTile(
            title: Text("TimeLine"),
            onTap: () {
              if (controller.selectedChannel.value != null) {
                Get.toNamed(Routes.timeLine.toString(),
                    arguments: controller.selectedChannel.value);
              } else {
                showErrorMessage("Please Select a Channel First");
              }
            },
          ),
          Divider(),
          ListTile(
            title: Text("Update Nick Name"),
            subtitle: Obx(
              () => AppTextField(
                hintText: '',
                maxLines: 1,
                characterLimit: 20,
                text: controller.name.value,
                onChanged: (s) {
                  controller.name.value = s;
                },
              ),
            ),
            trailing: SizedBox(
              child: Obx(() => controller.saveLoading.value
                  ? CircularProgressIndicator()
                  : FloatingActionButton(
                      onPressed: () {
                        controller.setName();
                      },
                      mini: true,
                      child: Icon(Icons.save),
                    )),
              width: 60,
              height: 60,
            ),
          )
        ],
      ),
    );
  }
}

class HomeController extends GetxController {
  final platformVersion = 'Unknown'.obs;
  final selectedChannel = Rx<Channel?>(null);
  final accessToken = Rx<String?>(null);

  @override
  void onInit() {
    super.onInit();
    if (kIsWeb) {
      platformVersion.value = "Web";
    } else {
      if (Platform.isAndroid) {
        platformVersion.value = "Android";
      } else if (Platform.isIOS) {
        platformVersion.value = "IOS";
      } else if (Platform.isFuchsia) {
        platformVersion.value = "Fuchsia";
      } else if (Platform.isLinux) {
        platformVersion.value = "Linux";
      } else if (Platform.isMacOS) {
        platformVersion.value = "MacOS";
      } else if (Platform.isWindows) {
        platformVersion.value = "Windows";
      }
    }
  }

  final accessTokenLoading = false.obs;
  final name = "".obs;

  @override
  void onReady() async {
    super.onReady();
    final sharedPref = await Get.putAsync(() => SharedPreferences.getInstance(),
        permanent: true);

    final sdk = Get.put<EngagementSDK>(
        EngagementSDK.accessToken("8PqSNDgIVHnXuJuGte1HdvOjOqhCFE1ZCR3qhqaS",
            sharedPref.getString("accessToken"), (event) async {
          print("Access Token: $event");
          await sharedPref.setString("accessToken", event);
          accessToken.value = event;
          accessTokenLoading.value = false;
        }),
        permanent: true);
    accessToken.value = sharedPref.getString("accessToken");
    if (accessToken.value == null) accessTokenLoading.value = true;
    sdk.errorStream.listen((error) {
      print("Error: $error");
      accessTokenLoading.value = false;
    });
    getName();
  }

  void getName() async {
    final sdk = Get.find<EngagementSDK>();
    name.value = await sdk.getNickName();
  }

  final saveLoading = false.obs;

  void setName() async {
    final sdk = Get.find<EngagementSDK>();
    saveLoading.value = true;
    await sdk.updateNickName(name.value);
    saveLoading.value = false;
  }
}

class Channel {
  String? name;
  String? id;
  String? stream_url;
}
