import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk_example/app_widgets.dart';

import 'home.dart';

class WidgetDetails extends StatelessWidget {
  final controller = Get.put(WidgetDetailsController());

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(),
      body: Padding(
        padding: const EdgeInsets.all(10),
        child: Column(
          children: [
            Expanded(
              child: Obx(() => controller.customWidget.value != null
                  ? Card(
                      child: Column(
                        children: [
                          Text(
                              "${controller.customWidget.value!.liveLikeWidget.question}"),
                          Padding(padding: EdgeInsets.all(5)),
                          controller.customWidget.value!.liveLikeWidget.kind ==
                                  'alert'
                              ? ListTile(
                                  title: Text(
                                      "${controller.customWidget.value!.liveLikeWidget.title}"),
                                  subtitle: Text(
                                      "${controller.customWidget.value!.liveLikeWidget.text} \n ${controller.customWidget.value!.liveLikeWidget.image_url} \n${controller.customWidget.value!.liveLikeWidget.link_label} \n${controller.customWidget.value!.liveLikeWidget.link_url}"),
                                )
                              : controller.customWidget.value!.liveLikeWidget
                                          .kind ==
                                      "emoji-slider"
                                  ? Column(
                                      children: [
                                        Slider(
                                          min: 0,
                                          max: 1,
                                          value: controller.sliderVal.value,
                                          onChanged: (value) {
                                            controller.sliderVal.value = value;
                                          },
                                          onChangeEnd: (value) {
                                            print("Vote>> $value");
                                            controller.customWidget.value!
                                                .vote(magnitude: value);
                                          },
                                        ),
                                        Container(
                                          child: Text(
                                              "${controller.customWidget.value!.liveLikeWidget.average_magnitude}"),
                                        )
                                      ],
                                    )
                                  : Column(
                                      children: (controller.customWidget.value!
                                                      .liveLikeWidget.options ??
                                                  controller.customWidget.value!
                                                      .liveLikeWidget.choices)
                                              ?.map((op) => InkWell(
                                                    child: Container(
                                                      padding:
                                                          const EdgeInsets.all(
                                                              10),
                                                      margin:
                                                          const EdgeInsets.all(
                                                              10),
                                                      decoration: BoxDecoration(
                                                          border: Border.all(
                                                              color:
                                                                  Colors.grey)),
                                                      child: Row(
                                                        children: [
                                                          Expanded(
                                                              child: Text(
                                                                  "${op.description}\n${op.id}")),
                                                          Text(
                                                              "${op.vote_count ?? op.answer_count}")
                                                        ],
                                                      ),
                                                    ),
                                                    onTap: () {
                                                      controller
                                                          .customWidget.value!
                                                          .vote(
                                                              optionId: op.id);
                                                    },
                                                  ))
                                              .toList() ??
                                          [],
                                    )
                        ],
                      ),
                    )
                  : controller.session.value != null
                      ? WidgetView(
                          session: controller.session.value!,
                          key: controller.widgetKey,
                          onWidgetViewInit: () {},
                          widgetListener: (widgetId, widgetKind) {
                            print("object>> $widgetId => $widgetKind");
                          },
                        )
                      : SizedBox.shrink()),
            ),
            AppTextField(
              hintText: 'WidgetId',
              onChanged: (s) {
                controller.widgetId = s;
              },
              text: controller.widgetId,
            ),
            AppTextField(
              hintText: 'WidgetKind',
              onChanged: (s) {
                controller.widgetKind = s;
              },
              text: controller.widgetKind,
            ),
            Padding(padding: EdgeInsets.all(10)),
            Wrap(
              children: [
                AppRaisedButton(
                  title: 'Fetch',
                  onPressed: () {
                    controller.getDetails();
                  },
                ),
                Padding(padding: EdgeInsets.all(10)),
                AppRaisedButton(
                  title: 'Fetch User Interaction',
                  onPressed: () {
                    controller.fetchWidgetInteraction();
                  },
                ),
                Padding(padding: EdgeInsets.all(10)),
                AppRaisedButton(
                  title: 'display Widget',
                  onPressed: () {
                    controller.displayWidget();
                  },
                ),
                Padding(padding: EdgeInsets.all(10)),
                AppRaisedButton(
                  title: 'custom Widget',
                  onPressed: () {
                    controller.showCustomWidget();
                  },
                )
              ],
            ),
            Expanded(
                child: Padding(
              padding: const EdgeInsets.all(10),
              child: Obx(
                () => controller.widgetLoading.value
                    ? Center(child: CircularProgressIndicator())
                    : controller.widget.value?.id == null
                        ? Text("No Widget Data")
                        : ListView(
                            padding: const EdgeInsets.all(5),
                            children: [
                              ListTile(
                                title: Text("id"),
                                subtitle:
                                    Text("${controller.widget.value?.id}"),
                                trailing: IconButton(
                                    onPressed: () {
                                      Clipboard.setData(ClipboardData(
                                          text: controller.widget.value?.id!));
                                    },
                                    icon: Icon(Icons.copy)),
                              ),
                              ListTile(
                                title: Text("kind"),
                                subtitle:
                                    Text("${controller.widget.value?.kind}"),
                              ),
                              ListTile(
                                title: Text("options"),
                                subtitle: Column(
                                  children: (controller.widget.value?.options ??
                                              controller.widget.value?.choices)
                                          ?.map((e) => Text(
                                              "-------\nId: ${e.id} \nDesc: ${e.description} \nVote Count: ${e.vote_count}\nAnswer Count: ${e.answer_count} -------"))
                                          .toList() ??
                                      <Widget>[],
                                ),
                              ),
                              ListTile(
                                title: Text("user interaction"),
                                subtitle: Text(
                                  "is Correct: ${controller.widgetInteraction.value?.is_correct} \nChoice Id: ${controller.widgetInteraction.value?.choice_id} \nMagnitude: ${controller.widgetInteraction.value?.magnitude} \nOption Id:${controller.widgetInteraction.value?.option_id} \nVote Count: ${controller.widgetInteraction.value?.vote_count}",
                                ),
                              )
                            ],
                          ),
              ),
            ))
          ],
        ),
      ),
    );
  }
}

class WidgetDetailsController extends GetxController {
  final sdk = Get.find<EngagementSDK>();

  String? widgetId = "1c5cdd27-3bdf-487b-a6b4-3fb15138071b";
  String? widgetKind = "text-prediction-follow-up";
  final widgetLoading = false.obs;
  final widgetKey = GlobalKey<WidgetViewState>();
  final widgetInteraction = Rx<WidgetUserInteraction?>(null);
  final customWidget = Rx<WidgetModel?>(null);
  final widget = Rx<LiveLikeWidget?>(null);
  final session = Rx<Session?>(null);
  final sliderVal = 0.0.obs;

  @override
  void onInit() async {
    super.onInit();

    final Channel channel = Get.arguments;
    session.value = await sdk.createSession(channel.id!);
  }

  void getDetails() async {
    if (widgetId == null || widgetId?.isEmpty == true)
      return showErrorMessage("WidgetId cannot be null or empty");
    if (widgetKind == null || widgetKind?.isEmpty == true)
      return showErrorMessage("widgetKind cannot be null or empty");
    widgetLoading.value = true;
    widget.value = await sdk.fetchWidgetDetails(widgetId!, widgetKind!);
    widgetLoading.value = false;
  }

  void fetchWidgetInteraction() async {
    if (session.value != null && widgetId != null && widgetKind != null) {
      widgetInteraction.value =
          await sdk.fetchWidgetInteraction(widgetId!, widgetKind!);
    }
  }

  void displayWidget() async {
    if (widget.value != null) {
      await widgetKey.currentState?.enableDefaultWidgetTransition(false);
      await widgetKey.currentState?.showDismissButton(false);
      widgetKey.currentState?.displayWidget(widget.value!);
      await Future.delayed(const Duration(seconds: 4));
      await widgetKey.currentState?.setWidgetState(WidgetState.results);
    }
  }

  void showCustomWidget() async {
    final data = widget.value;
    if (data != null) {
      final widgetModel = await session.value!.fetchWidgetModel(data);
      widgetModel?.voteCountResultStream.listen((event) {
        final index = customWidget.value?.liveLikeWidget.options
            ?.indexWhere((element) => element.id == event.optionId);
        final index2 = customWidget.value?.liveLikeWidget.options
            ?.indexWhere((element) => element.id == event.choiceId);
        if (index != null && index != -1) {
          customWidget.value?.liveLikeWidget.options?[index].vote_count =
              event.voteCount;
          customWidget.value?.liveLikeWidget.average_magnitude =
              event.averageMagnitude.toString();
          customWidget.refresh();
        }
        if (index2 != null && index2 != -1) {
          customWidget.value?.liveLikeWidget.options?[index2].answer_count =
              event.answerCount;
          customWidget.refresh();
        }
      });
      customWidget.value = widgetModel;
    }
  }
}
