import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk_example/app_widgets.dart';

import 'home.dart';

class WidgetJsonThemeScreen extends StatelessWidget {
  final _controller = Get.put(WidgetJsonThemeController());

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(),
      body: Column(
        children: [
          Obx(() => AppTextField(
                hintText: 'Enter Json Theme String',
                maxLines: 5,
                minLines: 5,
                text: _controller.jsonString.value,
                onChanged: (s) {
                  _controller.jsonString.value = s;
                },
              )),
          Obx(() => _controller.allowTheme.value
              ? ElevatedButton(
                  onPressed: () {
                    _controller.setTheme();
                  },
                  child: Text("Set Theme"),
                )
              : SizedBox.shrink()),
          Expanded(
              child: Obx(
            () => _controller.session.value != null
                ? Padding(
                    padding: const EdgeInsets.all(15),
                    child: WidgetView(
                      key: _controller.widgetKey,
                      onWidgetViewInit: () {
                        _controller.allowTheme.value = true;
                      },
                      session: _controller.session.value!,
                    ),
                  )
                : Center(child: CircularProgressIndicator()),
          ))
        ],
      ),
    );
  }
}

class WidgetJsonThemeController extends GetxController {
  final jsonString = "".obs;
  final sdk = Get.find<EngagementSDK>();
  final widgetKey = GlobalKey<WidgetViewState>();
  final allowTheme = false.obs;
  final session = Rx<Session?>(null);

  @override
  void onReady() async {
    super.onReady();
    final Channel channel = Get.arguments;
    session.value = await sdk.createSession(channel.id!);
    String data = await DefaultAssetBundle.of(Get.context!)
        .loadString("assets/customWidgetThemeUpdated.json");
    jsonString.value = data;
  }

  void setTheme() async {
    String json = jsonString.value;
    try {
      await widgetKey.currentState?.setJsonTheme(json);
    } on PlatformException catch (e) {
      if (e.message != null) showErrorMessage(e.message!);
    }
  }
}
