import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk/src/constants.dart';

class TimeLineView extends StatefulWidget {
  final Session session;
  final double sepratorHeight;
  final Function()? onInit;

  const TimeLineView(
      {Key? key, required this.session, this.onInit, this.sepratorHeight = 0})
      : super(key: key);

  @override
  _TimeLineViewState createState() => _TimeLineViewState();
}

class _TimeLineViewState extends State<TimeLineView> {
  MethodChannel? _channel;
  final Map<String, dynamic> creationParams = <String, dynamic>{};

  @override
  Widget build(BuildContext context) {
    return Platform.isAndroid
        ? PlatformViewLink(
            surfaceFactory: (context, controller) {
              return AndroidViewSurface(
                controller: controller as AndroidViewController,
                gestureRecognizers: const <
                    Factory<OneSequenceGestureRecognizer>>{},
                hitTestBehavior: PlatformViewHitTestBehavior.opaque,
              );
            },
            onCreatePlatformView: (params) {
              return PlatformViewsService.initSurfaceAndroidView(
                id: params.id,
                viewType: timeLineViewTypeKey,
                layoutDirection: TextDirection.ltr,
                creationParams: creationParams,
                creationParamsCodec: const StandardMessageCodec(),
              )
                ..addOnPlatformViewCreatedListener((id) async {
                  _channel = new MethodChannel('$timeLineViewTypeKey$id');
                  await _channel?.invokeMethod(setSessionKey, {
                    programIDKey: widget.session.programId,
                    "gap": widget.sepratorHeight
                  });
                  params.onPlatformViewCreated(id);
                  widget.onInit?.call();
                })
                ..create();
            },
            viewType: widgetViewTypeKey)
        : Platform.isIOS
            ? UiKitView(
                viewType: timeLineViewTypeKey,
                layoutDirection: TextDirection.ltr,
                creationParams: creationParams,
                creationParamsCodec: const StandardMessageCodec(),
                gestureRecognizers: const <
                    Factory<OneSequenceGestureRecognizer>>{},
                hitTestBehavior: PlatformViewHitTestBehavior.opaque,
                onPlatformViewCreated: (id) async {
                  _channel = new MethodChannel('$timeLineViewTypeKey$id');
                  await _channel?.invokeMethod(setSessionKey, {
                    programIDKey: widget.session.programId,
                    "gap": widget.sepratorHeight
                  });
                  widget.onInit?.call();
                },
              )
            : SizedBox.shrink();
  }
}
