import 'package:flutter/services.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk/src/constants.dart';

class WidgetModel {
  LiveLikeWidget liveLikeWidget;
  late EventChannel _eventChannel;

  WidgetModel(this.liveLikeWidget, String streamChannelKey) {
    _eventChannel = EventChannel(streamChannelKey);
  }

  Future<void> vote({String? optionId, double? magnitude}) async {
    await livelike_method_channel.invokeMethod("submitVoteToWidgetModel", {
      "widgetId": liveLikeWidget.id!,
      "optionId": optionId,
      "magnitude": magnitude
    });
  }

  Future<void> loadInteractionHistory() async {
    await livelike_method_channel.invokeMethod(
        "getUserInteractionHistoryForWidgetModel",
        {"widgetId": liveLikeWidget.id!});
  }

  Stream<VoteCountResult> get voteCountResultStream => _eventChannel
      .receiveBroadcastStream()
      .map((event) => VoteCountResult.fromJson(event));

  Future<void> dispose() async {
    await livelike_method_channel
        .invokeMethod("disposeWidgetModel", {"widgetId": liveLikeWidget.id!});
  }
}

class VoteCountResult {
  String? optionId;
  int? voteCount;
  double? averageMagnitude;
  String? choiceId;
  int? answerCount;

  VoteCountResult.fromJson(dynamic data) {
    optionId = data["optionId"];
    voteCount = data["voteCount"];
    averageMagnitude = data["averageMagnitude"];
    choiceId = data["choiceID"];
    answerCount = data["answerCount"];
  }
}
